/*
 * Copyright (C) 2014 Víctor Matía Rodríguez.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import QtQuick.Layouts 1.1

import Ubuntu.Components 1.1
import Ubuntu.Components.Pickers 1.0
import Ubuntu.Components.Popups 1.0

import "../../"

Item {
    width: parent.width
    height: parent.height

    // In case the fields are not all filled, complain
    EmptyFieldsFailComponent {
        id: fieldsFailDialog
        objectName: "fieldsFailDialog"
    }

    // In case the passwords are not equal, complain
    BadPassesComponent {
        id: passFailDialog
        objectName: "passFailDialog"
    }

    // The shape that contains the form
    UbuntuShape {
        id: newUserFormShape
        objectName: "newUserFormShape"

        width: parent.width - root.margins * 2
        height: parent.height - root.margins * 2 - root.spacing * 2 -
                submitBtn.height
        x: root.margins
        y: root.margins

        // With a flickable so it can be moved in case the screen is small
        Flickable {
            id: newUserFormFlickable
            objectName: "newUserFormFlickable"

            clip: true
            width: parent.width
            height: parent.height
            contentWidth: parent.width
            contentHeight: grid.height + root.spacing * 2

            // And the grid that contains the form itself
            GridLayout {
                id: grid
                objectName: "grid"

                columnSpacing: root.spacing
                rowSpacing: root.spacing
                columns: 2
                rows: 8
                width: parent.width - root.spacing * 2
                x: newUserFormFlickable.x + root.spacing
                y: newUserFormFlickable.y + root.spacing

                Label {
                    id: nameLbl
                    objectName: "nameLbl"

                    text: i18n.tr("Name:")
                }

                TextField {
                    id: nameField
                    objectName: "nameField"

                    Layout.fillWidth: true
                    placeholderText: i18n.tr("My Name")
                    inputMethodHints: {
                        Qt.ImhNoPredictiveText
                        Qt.ImhSensitiveData
                    }
                }

                Label {
                    id: surnameLbl
                    objectName: "surnameLbl"

                    text: i18n.tr("Surname:")
                }

                TextField {
                    id: surnameField
                    objectName: "surnameField"

                    Layout.fillWidth: true
                    placeholderText: i18n.tr("My Surname")
                    inputMethodHints: {
                        Qt.ImhNoPredictiveText
                        Qt.ImhSensitiveData
                    }
                }

                Label {
                    id: idLbl
                    objectName: "idLbl"

                    text: i18n.tr("ID:")
                }

                TextField {
                    id: idField
                    objectName: "idField"

                    Layout.fillWidth: true
                    placeholderText: i18n.tr("My ID")
                    inputMethodHints: {
                        Qt.ImhNoAutoUppercase
                        Qt.ImhNoPredictiveText
                        Qt.ImhSensitiveData
                    }
                }

                Label {
                    id: birthDateLbl
                    objectName: "birthDateLbl"

                    text: i18n.tr("Birth date:")
                }

                DatePicker {
                    id: birthDatePicker
                    objectName: "birthDatePicker"

                    Layout.fillWidth: true
                    maximum: new Date()
                    minimum: new Date("1900/01/01")
                }

                Label {
                    id: emailLbl
                    objectName: "emailLbl"

                    text: i18n.tr("E-Mail:")
                }

                TextField {
                    id: emailField
                    objectName: "emailField"

                    Layout.fillWidth: true
                    placeholderText: i18n.tr("my@mail.com")
                    inputMethodHints: {
                        Qt.ImhEmailCharactersOnly
                        Qt.ImhNoAutoUppercase
                        Qt.ImhNoPredictiveText
                        Qt.ImhSensitiveData
                    }
                }

                Label {
                    id: firstPassLbl
                    objectName: "firstPassLbl"

                    text: i18n.tr("Password:")
                }

                TextField {
                    id: firstPassField
                    objectName: "firstPassField"

                    Layout.fillWidth: true
                    placeholderText: i18n.tr("My Password")
                    echoMode: TextInput.Password
                    inputMethodHints: {
                        Qt.ImhNoAutoUppercase
                        Qt.ImhNoPredictiveText
                        Qt.ImhSensitiveData
                    }

                    onTextChanged: {
                        // If the passwords are not the same
                        if (text !== secondPassField.text)
                            // The text is red
                            secondPassField.color = UbuntuColors.red
                        else
                            // Back to normal
                            secondPassField.color = UbuntuColors.darkGrey
                    }
                }

                Label {
                    id: secondPassLbl
                    objectName: "secondPassLbl"

                    text: i18n.tr("Repeat password:")
                }

                TextField {
                    id: secondPassField
                    objectName: "secondPassField"

                    Layout.fillWidth: true
                    placeholderText: i18n.tr("My Password")
                    echoMode: TextInput.Password
                    inputMethodHints: {
                        Qt.ImhNoAutoUppercase
                        Qt.ImhNoPredictiveText
                        Qt.ImhSensitiveData
                    }

                    onTextChanged: {
                        // If the passwords are not the same
                        if (firstPassField.text !== text)
                            color = UbuntuColors.red // The text is red
                        else
                            color = UbuntuColors.darkGrey // Back to normal
                    }
                }

                Label {
                    id: showPassLbl
                    objectName: "showPassLbl"

                    text: i18n.tr("Show passwords:")
                }

                // Show the passwords
                CheckBox {
                    id: showPassCheckBox
                    objectName: "showPassCheckBox"

                    Layout.alignment: Layout.Center
                    text: i18n.tr("Show passwords:")

                    onCheckedChanged: {
                        if (checked)
                        {
                            firstPassField.echoMode = TextInput.Normal
                            secondPassField.echoMode = TextInput.Normal
                        }
                        else
                        {
                            firstPassField.echoMode = TextInput.Password
                            secondPassField.echoMode = TextInput.Password
                        }
                    }
                }
            }
        }
    }

    // The button to submit the request
    Button {
        id: submitBtn
        objectName: "submitBtn"

        x: parent.width / 2 - width / 2
        y: parent.height - height - root.margins
        text: i18n.tr("Submit")
        color: UbuntuColors.orange

        onClicked: {
            // In case the fields are empty, complain
            if (!nameField.text || !surnameField.text || !idField.text
                    || !emailField.text || !firstPassField.text
                    || !secondPassField.text)
                PopupUtils.open(fieldsFailDialog)
            // In case the passwords are not the same, complain
            else if (firstPassField.text !== secondPassField.text)
                PopupUtils.open(passFailDialog)
        }
    }
}
