/*
 * Copyright (C) 2014 Víctor Matía Rodríguez.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QDir>
#include <QFile>
#include <QIODevice>
#include <QStandardPaths>
#include <QTextStream>

#include "cheetahcs.h"

// The constructor
CheetaHCS::CheetaHCS(QObject *parent) :
    QObject(parent), m_configExists(true), m_hospital(""), m_primaryCenter(""),
    m_hospitals(), m_primaryCenters() // Initialize the Q_PROPERTIES
{
    // This is the config folder
    m_folder = QString(QStandardPaths::writableLocation(
                QStandardPaths::DataLocation));
    // And the config file
    m_file = QString(m_folder + "/location.conf");

    // The hospitals and primary centers lists
    m_hospitals << "El Bierzo" << "Serranía-Ronda";
    m_primaryCenters << "Picotuerto, Ponferrada II" <<
                        "Ronda-Sur, Santa Bárbara";
}

// The destructor
CheetaHCS::~CheetaHCS()
{

}

/*
 * Whether the config file exists or not, used to prompt with the configuration
 * menu
 */
bool CheetaHCS::configExists()
{
    return m_configExists;
}

// Change the state of the property
void CheetaHCS::setConfigExists(bool configExists)
{
    if (configExists != m_configExists)
    {
        m_configExists = configExists;
        Q_EMIT configExistsChanged(); // Emit the corresponding signal
    }
}

// The hospital that has been chosen
QString CheetaHCS::hospital()
{
    return m_hospital;
}

// Change the state of the property
void CheetaHCS::setHospital(QString hospital)
{
    if (hospital != m_hospital)
    {
        m_hospital = hospital;
        Q_EMIT hospitalChanged(); // Emit the corresponding signal
    }
}

// The primary center that has been chosen
QString CheetaHCS::primaryCenter()
{
    return m_primaryCenter;
}

// Change the state of the property
void CheetaHCS::setPrimaryCenter(QString primaryCenter)
{
    if (primaryCenter != m_primaryCenter)
    {
        m_primaryCenter = primaryCenter;
        Q_EMIT primaryCenterChanged(); // Emit the corresponding signal
    }
}

// The hospitals that are available
QStringList CheetaHCS::hospitals()
{
    return m_hospitals;
}

// Change the state of the property
void CheetaHCS::setHospitals(QStringList hospitals)
{
    if (hospitals != m_hospitals)
    {
        m_hospitals = hospitals;
        Q_EMIT hospitalsChanged(); // Emit the corresponding signal
    }
}

// The primary centers that are available
QStringList CheetaHCS::primaryCenters()
{
    return m_primaryCenters;
}

// Change the state of the property
void CheetaHCS::setPrimaryCenters(QStringList primaryCenters)
{
    if (primaryCenters != m_primaryCenters)
    {
        m_primaryCenters = primaryCenters;
        Q_EMIT primaryCentersChanged(); // Emit the correspongin signal
    }
}

// Run the start checks and read the configuration file if available
void CheetaHCS::startChecks()
{
    // Check if the folder exists and create it if not
    if (startChecksFunction.folderExists(m_folder) == false)
        QDir().mkpath(m_folder);

    // Check if the config file exists and create it if not
    if (startChecksFunction.configExists(m_file) == false)
        setConfigExists(false);

    // If the config file exists, read it
    if (QFile(m_file).exists())
        readConfiguration(m_file);
}

// Create the configuration file
void CheetaHCS::configure(int hospital, int primaryCenter) /* int for the
                                                            * selectedIndex
                                                            * of each */
{
    QFile file(m_file); // The config file

    QString hosp; // The hospital string
    QString pCenter; // The primary center string

    // For every hospital in the list
    for (int i = 0; i < m_hospitals.length(); i++)
    {
        // If the selected index is the one of the list
        if (hospital == i)
            hosp = m_hospitals[i]; // Set the string
    }

    // For every primary center in the list
    for (int i = 0; i < m_primaryCenters.length(); i++)
    {
        // If the selected index is the one of the list
        if (primaryCenter == i)
            pCenter = m_primaryCenters[i]; // Set the string
    }

    // Remove the file if it exists and rewrite it
    if (file.exists())
        QFile().remove(m_file);

    // If the config file can be open
    if (file.open(QIODevice::ReadWrite))
    {
        // Create a stream of text in the file
        QTextStream stream(&file);
        // Write the configuration to the file
        stream << "hospital=" << hosp << endl << "primaryCenter=" <<
                  pCenter << endl;
    }
}

// Read the configuration file
void CheetaHCS::readConfiguration(QString configFile)
{
    QFile file(configFile); // The file itself

    // If it can be open
    if (file.open(QIODevice::ReadOnly))
    {
        // Create a stream in the file
        QTextStream stream(&file);

        // While there are lines to read
        while (!stream.atEnd())
        {
            QString line = stream.readLine(); // Get the line
            QStringList fields = line.split("="); // Split it

            // Now get the properties for hospital and primary center
            if (fields[0] == "hospital")
                setHospital(fields[1]);
            else if (fields[0] == "primaryCenter")
                setPrimaryCenter(fields[1]);
        }
    }

    // Close the file
    file.close();
}

void CheetaHCS::readConfig()
{
    readConfiguration(m_file);
}
